<?php

namespace CBSD_App\Controllers;

if (!defined('ABSPATH')) {
    exit;
}

use WP_REST_Request;
use WP_REST_Response;

class CBSD_StatsController
{
    private $cache_ttl = 3600; // Time to live in seconds (1 hour)

    public function lineChartData()
    {
        global $wpdb;
        $cache_key = 'cbsd_line_chart_data';
        $cache_group = 'cbsd_stats';

        // Try to get cached data
        $data = wp_cache_get($cache_key, $cache_group);

        if ($data === false) {
            // Cache miss, query the database
            $query = $wpdb->prepare("SELECT MONTH(created_at) as month, type, COUNT(*) as total FROM {$wpdb->prefix}cbsd_logs GROUP BY MONTH(created_at), type");
            $data = $wpdb->get_results($query);

            // Store the data in cache
            wp_cache_set($cache_key, $data, $cache_group, $this->cache_ttl);
        }

        return new WP_REST_Response($data, 200);
    }

    public function threatsByPost()
    {
        global $wpdb;
        $cache_key = 'cbsd_threats_by_post';
        $cache_group = 'cbsd_stats';

        // Try to get cached data
        $data = wp_cache_get($cache_key, $cache_group);

        if ($data === false) {
            // Cache miss, query the database
            $query = $wpdb->prepare("SELECT post_id, COUNT(*) as attacks FROM {$wpdb->prefix}cbsd_logs WHERE type = %s OR type = %s GROUP BY post_id", 'XSS', 'SQLI');
            $data = $wpdb->get_results($query);

            // Map each result to include post title and total visits
            $data = array_map(function ($item) use ($wpdb) {
                $post = get_post($item->post_id);
                $item->attacks = (int) $item->attacks;
                $item->title = $post->post_title;
                $visit_query = $wpdb->prepare("SELECT COUNT(*) FROM {$wpdb->prefix}cbsd_visits WHERE post_id = %d", $item->post_id);
                $item->total_visits = (int) $wpdb->get_var($visit_query);
                return $item;
            }, $data);

            // Store the data in cache
            wp_cache_set($cache_key, $data, $cache_group, $this->cache_ttl);
        }

        return new WP_REST_Response($data, 200);
    }

    public function visitorsByCountry()
    {
        global $wpdb;
        $cache_key = 'cbsd_visitors_by_country';
        $cache_group = 'cbsd_stats';

        // Try to get cached data
        $data = wp_cache_get($cache_key, $cache_group);

        if ($data === false) {
            // Cache miss, query the database
            $query = $wpdb->prepare("SELECT country_id, COUNT(*) as visits FROM {$wpdb->prefix}cbsd_visits GROUP BY country_id");
            $data = $wpdb->get_results($query);

            // Get total visits
            $total_visits_query = $wpdb->prepare("SELECT COUNT(*) FROM {$wpdb->prefix}cbsd_visits");
            $totalVisits = (int) $wpdb->get_var($total_visits_query);

            // Map each result to include country name and visit percentage
            $data = array_map(function ($item) use ($wpdb, $totalVisits) {
                $country_query = $wpdb->prepare("SELECT country_name FROM {$wpdb->prefix}cbsd_country_codes WHERE id = %d", $item->country_id);
                $item->country = $wpdb->get_var($country_query);
                $item->visits = (int) $item->visits;
                $item->percentage = round(($item->visits / $totalVisits) * 100, 2);
                return $item;
            }, $data);

            // Store the data in cache
            wp_cache_set($cache_key, $data, $cache_group, $this->cache_ttl);
        }

        return new WP_REST_Response($data, 200);
    }

    public function currentYearVsLastByMonth()
    {
        global $wpdb;
        $cache_key = 'cbsd_current_vs_last_year_by_month';
        $cache_group = 'cbsd_stats';

        // Try to get cached data
        $data = wp_cache_get($cache_key, $cache_group);

        if ($data === false) {
            // Cache miss, get current year and last year data
            $currentYear = gmdate('Y');
            $lastYear = $currentYear - 1;

            $current_year_query = $wpdb->prepare("SELECT MONTH(created_at) as month, COUNT(*) as total FROM {$wpdb->prefix}cbsd_logs WHERE YEAR(created_at) = %d GROUP BY MONTH(created_at)", $currentYear);
            $currentYearData = $wpdb->get_results($current_year_query);

            $last_year_query = $wpdb->prepare("SELECT MONTH(created_at) as month, COUNT(*) as total FROM {$wpdb->prefix}cbsd_logs WHERE YEAR(created_at) = %d GROUP BY MONTH(created_at)", $lastYear);
            $lastYearData = $wpdb->get_results($last_year_query);

            $data = [
                'currentYear' => $currentYearData,
                'lastYear' => $lastYearData
            ];

            // Store the data in cache
            wp_cache_set($cache_key, $data, $cache_group, $this->cache_ttl);
        }

        return new WP_REST_Response($data, 200);
    }
}
