<?php
namespace CBSD_App\Types;

use CBSD\Classes\CBSD_CustomRulesEngine;
use function CBSD_Helpers\cbsd_banned;
use function CBSD_Helpers\cbsd_get_current_real_country;
use function CBSD_Helpers\cbsd_get_current_real_ip;
use function CBSD_Helpers\cbsd_log;
use function CBSD_Helpers\cbsd_redirect;
use function CBSD_Helpers\cbsd_setting;


class CBSD_App
{
    public CBSD_CustomRulesEngine $customRuleEngine;
    public $modules;
    public $underAttack;
    static public $config;
    static public $dashboardURL;

    function __construct($config)
    {
        static::$config = $config;
        static::$dashboardURL = $config['SERVER_BASE'];

        $this->check_ban();
        // load modules
        $this->modules = $this->load_modules(__DIR__ . '/../Modules');
        $this->underAttack = false;
        $this->customRuleEngine = new CBSD_CustomRulesEngine();
    }

    public function process_request($request)
    {
        // log a visit
        $this->logVisit();

        // call the each module on request
        foreach ($this->modules as $module) {
            $result = $module->main($request);
        }

        $whitelist = $this->getModInstance('Whitelist');
        $blacklist = $this->getModInstance('Blacklist');

        // check whitelist (user agent, ip, country)
        $whitelist->main($request);

        if ($request->shouldPass)
            return;
        else
            $blacklist->main($request);

        // check again after blacklist scan
        if (!$request->shouldPass)
            cbsd_log("BLACKLIST", null, fn() => wp_die(wp_kses("<div style=\"text-align: center;font-size: 20px; font-weight: bold;\">Your request has been blocked</div></div><footer style=\"font-size: 10px; margin-top: 2.5rem;\">Powered by: <a href=\"https://securas.fr\">Cybershield</a></footer>", array (
                'div' => array (
                    'style' => array (),
                ),
                'footer' => array (
                    'style' => array (),
                ),
                'a' => array (
                    'href' => array (),
                ),
            )), 403));

        if ($this->customRuleEngine->check($request)) {
            cbsd_log("CUSTOM_RULE", null, fn() => wp_die(wp_kses("<div style=\"text-align: center;font-size: 20px; font-weight: bold;font-size: 20px; font-weight: bold;\">Your request has been blocked</div></div><footer style=\"font-size: 10px; margin-top: 2.5rem;\">Powered by: <a href=\"https://securas.fr\">Cybershield</a></footer>", array (
                'div' => array (
                    'style' => array (),
                ),
                'footer' => array (
                    'style' => array (),
                ),
                'a' => array (
                    'href' => array (),
                ),
            )), 403));
        }
    }

    private function load_modules($path)
    {
        $modules = [];
        if ($dh = opendir($path)) {
            while (($file = readdir($dh)) !== false) {
                if ($file != '..' && $file != '.') {
                    $entry = static::$config['MODULE_INIT'];
                    $name = __DIR__ . '/../Modules/' . $file . '/' . $entry;
                    if ($file !== "Blacklist" && $file !== "Whitelist" && $file !== 'index.php') {
                        require_once $name;
                        $class = "CBSD_$file";
                        if (class_exists($class)) {
                            $module = new $class(static::$config);
                            $modules[] = $module;
                        }
                    }
                }
            }
            closedir($dh);
        }
        return $modules;
    }

    // // register modules in the database for the dashboard
    public function register_modules()
    {
        $api_key = get_option('api_key');

        foreach ($this->modules as $module) {
            // ...
        }
    }

    public function getModInstance($name)
    {
        $entry = static::$config['MODULE_INIT'];
        $class = __DIR__ . '/../Modules/' . $name . '/' . $entry;
        require_once $class;

        return new ("CBSD_$name")(static::$config);
    }

    public function register_settings()
    {
        $api_key = get_option('api_key');
        $messages = [];

        // ...

        return $messages;
    }

    private function check_ban()
    {
        $ip = cbsd_get_current_real_ip();
        $bannedStatus = cbsd_banned($ip);
        if ($bannedStatus !== false) {
            if (cbsd_setting('redirect_uri')) {
                cbsd_redirect(cbsd_setting('redirect_uri'));
            }
        }
        // Aucune redirection si l'IP n'est pas bannie (bannedStatus est null)
    }

    private function logVisit()
    {
        add_action('wp', function () {
            global $wpdb;
            $ip = cbsd_get_current_real_ip();
            $user_agent = sanitize_text_field(wp_unslash($_SERVER['HTTP_USER_AGENT'] ?? ''));
            $post_id = get_the_ID();
            $country = cbsd_get_current_real_country() ?? "TN";

            $query = $wpdb->prepare(
                "SELECT id FROM {$wpdb->prefix}cbsd_country_codes WHERE country_code = %s",
                $country
            );

            $country_id = $wpdb->get_var($query);

            $data = [
                'ip' => $ip,
                'user_agent' => $user_agent,
                'post_id' => $post_id,
                'created_at' => gmdate('Y-m-d H:i:s'),
                'country_id' => $country_id
            ];

            $wpdb->insert(
                "{$wpdb->prefix}cbsd_visits",
                $data,
                ['%s', '%s', '%d', '%s', '%d']
            );
        });
    }

    static public function config()
    {
        return self::$config;
    }

    public function setIsUnderAttack($value)
    {
        $this->underAttack = $value;
    }

}
